==> beta1.7.tar.bz2: beta1.7/class-chan-actions.cpp <==
/**************************************************
*  This file is a part of psotnic irc bot code
*  Copyright 2003 Grzegorz Rusin <pks@irc.pl>
*
*  PRIVATE CODE - DO NOT DISTRIBUTE
*
*/

#include "prots.h"
#include "global-var.h"

/* STRANGE ACTIONS */
void chan::updateLimit()
{
	char buf[MAX_LEN];
	int tolerance;

	if(nextlimit == -1 || !synced()) return;
	if(chset->LIMIT_TOLERANCE > 0) tolerance = chset->LIMIT_TOLERANCE;
	else tolerance = (chset->LIMIT_TOLERANCE * chset->LIMIT_OFFSET)/(-100);

	if(chset->LIMIT && nextlimit <= NOW && me->flags & IS_OP &&
		myTurn(chset->LIMIT_BOTS))
	{
		if((limit <= users.entries() + chset->LIMIT_OFFSET - tolerance) ||
			(limit > users.entries() + chset->LIMIT_OFFSET + tolerance))
		{
			sprintf(buf, "%d", users.entries() + chset->LIMIT_OFFSET);
			net.irc.send("MODE ", name, " +l ", buf, NULL);
			penalty += 4;
		}
		nextlimit = NOW + chset->LIMIT_TIME * chset->LIMIT_BOTS;
	}
}

void chan::enforceLimits()
{
	if(chset->ENFORCE_LIMITS && limit)
	{
		int n = users.entries() - limit;
		ptrlist<chanuser>::link *u = users.start();

		if(n > 0)
		{
			while(n && u)
			{
				if(!(u->ptr()->flags & HAS_F))
				{
					toKick.sortAdd(u->ptr());
					--n;
				}
				u = u->next();
			}
		}
	}
}

/* MULTIUSER ACTIONS */
int chan::massKick(int who)
{
	ptrlist<chanuser>::link *u = users.start();
	int i = 0;

	while(u)
	{
		if(!(u->ptr()->flags & HAS_F))
		{
			if((who & MK_OPS && u->ptr()->flags & IS_OP) || (who & MK_NONOPS && !(u->ptr()->flags & IS_OP)))
			{
				toKick.sortAdd(u->ptr());
				++i;
			}
		}
		u = u->next();
	}
	return i;
}

int chan::op(chanuser **MultHandle, int num)
{
	char *a = NULL;
	int i, j=0;

	if(!num) return 0;

	if(penalty < 10)
	{
		for(i=j=0; i<num; ++i)
		{
			if(!(MultHandle[i]->flags & OP_SENT))
			{
				a = push(a, " ", MultHandle[i]->nick, NULL);
   				MultHandle[i]->flags += OP_SENT;
				++j;
			}
		}
		if(j)
		{
			net.irc.send("MODE ", name, j == 3 ? " +ooo" : j == 2 ? " +oo" : " +o", a, NULL);
			penalty += actionPenalty(j);
			free(a);
		}
	}
	return j;
}

int chan::kick4(chanuser **MultHandle, int num)
{
	if(num < 1) return 0;
	if(num > 4) num = 4;

	char *a = NULL;
	int i, j=0;

	if(penalty < 10)
	{
		for(i=j=0; i<num; ++i)
		{
			if(*MultHandle[i]->nick && !(MultHandle[i]->flags & KICK_SENT))
			{
				a = push(a, a ? (char *) "," : (char *) " ", MultHandle[i]->nick, NULL);
				MultHandle[i]->flags |= KICK_SENT;
				++j;
			}
		}
		if(j)
		{
			net.irc.send("KICK ", name, a, " :", config.kickreason, NULL);
			penalty += actionPenalty(j);
			free(a);
			sentKicks += j;
		}
	}
	return j;
}

int chan::kick6(chanuser **MultHandle, int num)
{
	if(num < 1) return 0;
	if(num < 5 || penalty > 5) return kick4(MultHandle, num);
	else
	{
		if(num > 6) num = 6;

		if(penalty < 2)
			return kick4(MultHandle, 2) + kick4(MultHandle+2, num-2);
		else
			return kick4(MultHandle, 1) + kick4(MultHandle+1, num-1);
	}
	return 0;
}

/* SINGLEUSER */
int chan::op(chanuser *p)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & OP_SENT))
	{
		if(penalty < 10)
		{
			net.irc.send("MODE ", name, " +o ", p->nick, NULL);
			p->flags |= OP_SENT;
			penalty += 4;
			return 1;
		}
	}
	return 0;
}

int chan::deOp(chanuser *p)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & DEOP_SENT))
	{
		if(penalty < 10)
		{
			net.irc.send("MODE ", name, " -o ", p->nick, NULL);
			p->flags |= DEOP_SENT;
			penalty += 4;
			return 1;
		}
	}
	return 0;
}


int chan::kick(chanuser *p, char *reason)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & KICK_SENT))
	{
		if(penalty < 10)
		{
			if(reason) net.irc.send("KICK ", name, " ", p->nick, " :", reason, NULL);
			else net.irc.send("KICK ", name, " ", p->nick, " :", config.kickreason, NULL);
			penalty += 4;
			p->flags |= KICK_SENT;
			++sentKicks;
			return 1;
		}
	}
	return 0;
}

int chan::kickBan(chanuser *p, char *mask, char *reason)
{
	/*
	if(ME.NextAction <= NOW)
	{
		ME.NextAction = NOW;
		if(!(p->flags & BAN_SENT))
		{
			net.irc.send("MODE ", name, " +b ", mask, NULL);
			p->flags += BAN_SENT;
			++ME.NextAction;
		}
		if(!(p->flags & KICK_SENT))
		{
			if(reason) net.irc.send("KICK ", name, " ", p->nick, " :", reason, NULL);
			else net.irc.send("KICK ", name, " ", p->nick, " :", config.banreason, NULL);
			p->flags += KICK_SENT;
			ME.NextAction += AdditionalPenality();
		}
	}
	*/
	return 0;
}

int chan::invite(char *nick)
{
	if(!nick || !*nick) return 0;

	if(!getUser(nick))
	{
		char *a = strchr(nick, '!');
		if(a) *a = '\0';

		::invite.wisePush("INVITE ", nick, " ", name, NULL);
		if(penalty < 5) if(::invite.flush(&net.irc)) penalty += 3;
		return 1;
	}
	return 0;
}

==> beta2.0rc10.tar.bz2: beta2.0rc10/class-chan-actions.cpp <==
/**************************************************
*  This file is a part of psotnic irc bot code
*  Copyright 2003-2004 Grzegorz Rusin <pks@irc.pl>
*
*  PRIVATE CODE - DO NOT DISTRIBUTE
*
*/

#include "prots.h"
#include "global-var.h"

/* STRANGE ACTIONS */
void chan::updateLimit()
{
	char buf[MAX_LEN];
	int tolerance;

	if(nextlimit == -1 || !synced()) return;
	if(chset->LIMIT_TOLERANCE > 0) tolerance = chset->LIMIT_TOLERANCE;
	else tolerance = (chset->LIMIT_TOLERANCE * chset->LIMIT_OFFSET)/(-100);

	if(chset->LIMIT && nextlimit <= NOW && me->flags & IS_OP &&
		myTurn(chset->LIMIT_BOTS))
	{
		if((limit <= users.entries() + chset->LIMIT_OFFSET - tolerance) ||
			(limit > users.entries() + chset->LIMIT_OFFSET + tolerance))
		{
			sprintf(buf, "%d", users.entries() + chset->LIMIT_OFFSET);
			net.irc.send("MODE ", name, " +l ", buf, NULL);
			penalty += 4;
		}
		nextlimit = NOW + chset->LIMIT_TIME * chset->LIMIT_BOTS;
	}
}

void chan::enforceLimits()
{
	if(chset->ENFORCE_LIMITS && limit && synced())
	{
		int n = users.entries() - limit;
		ptrlist<chanuser>::link *u = users.start();

		if(n > 0)
		{
			while(n && u)
			{
				if(!(u->ptr()->flags & HAS_F))
				{
					toKick.sortAdd(u->ptr());
					--n;
				}
				u = u->next();
			}
		}
	}
}

/* MULTIUSER ACTIONS */
int chan::massKick(int who)
{
	ptrlist<chanuser>::link *u = users.start();
	int i = 0;

	while(u)
	{
		if(!(u->ptr()->flags & HAS_F))
		{
			if((who & MK_OPS && u->ptr()->flags & IS_OP) || (who & MK_NONOPS && !(u->ptr()->flags & IS_OP)))
			{
				toKick.sortAdd(u->ptr());
				++i;
			}
		}
		u = u->next();
	}
	return i;
}

int chan::op(chanuser **MultHandle, int num)
{
	char *a = NULL;
	int i, j=0;

	if(!num) return 0;

	if(penalty < 10)
	{
		for(i=j=0; i<num; ++i)
		{
			if(!(MultHandle[i]->flags & OP_SENT))
			{
				a = push(a, " ", MultHandle[i]->nick, NULL);
   				MultHandle[i]->flags += OP_SENT;
				++j;
			}
		}
		if(j)
		{
			net.irc.send("MODE ", name, j == 3 ? " +ooo" : j == 2 ? " +oo" : " +o", a, NULL);
			penalty += actionPenalty(j);
			free(a);
		}
	}
	return j;
}

int chan::kick4(chanuser **MultHandle, int num)
{
	if(num < 1) return 0;
	if(num > 4) num = 4;

	char *a = NULL;
	int i, j=0;

	if(penalty < 10)
	{
		for(i=j=0; i<num; ++i)
		{
			if(*MultHandle[i]->nick && !(MultHandle[i]->flags & KICK_SENT))
			{
				a = push(a, a ? (char *) "," : (char *) " ", MultHandle[i]->nick, NULL);
				MultHandle[i]->flags |= KICK_SENT;
				++j;
			}
		}
		if(j)
		{
			net.irc.send("KICK ", name, a, " :", config.kickreason, NULL);
			penalty += actionPenalty(j);
			free(a);
			sentKicks += j;
		}
	}
	return j;
}

int chan::kick6(chanuser **MultHandle, int num)
{
	if(num < 1) return 0;
	if(num < 5 || penalty > 5) return kick4(MultHandle, num);
	else
	{
		if(num > 6) num = 6;

		if(penalty < 2)
			return kick4(MultHandle, 2) + kick4(MultHandle+2, num-2);
		else
			return kick4(MultHandle, 1) + kick4(MultHandle+1, num-1);
	}
	return 0;
}

/* SINGLEUSER */
int chan::op(chanuser *p)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & OP_SENT))
	{
		if(penalty < 10)
		{
			net.irc.send("MODE ", name, " +o ", p->nick, NULL);
			p->flags |= OP_SENT;
			penalty += 4;
			return 1;
		}
	}
	return 0;
}

int chan::deOp(chanuser *p)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & DEOP_SENT))
	{
		if(penalty < 10)
		{
			net.irc.send("MODE ", name, " -o ", p->nick, NULL);
			p->flags |= DEOP_SENT;
			penalty += 4;
			return 1;
		}
	}
	return 0;
}


int chan::kick(chanuser *p, char *reason)
{
	if(!p || !p->nick || !*p->nick) return 0;

	if(!(p->flags & KICK_SENT))
	{
		if(penalty < 10)
		{
			if(reason) net.irc.send("KICK ", name, " ", p->nick, " :", reason, NULL);
			else net.irc.send("KICK ", name, " ", p->nick, " :", config.kickreason, NULL);
			penalty += 4;
			p->flags |= KICK_SENT;
			++sentKicks;
			return 1;
		}
	}
	return 0;
}

int chan::kickBan(chanuser *p, char *mask, char *reason)
{
	/*
	if(ME.NextAction <= NOW)
	{
		ME.NextAction = NOW;
		if(!(p->flags & BAN_SENT))
		{
			net.irc.send("MODE ", name, " +b ", mask, NULL);
			p->flags += BAN_SENT;
			++ME.NextAction;
		}
		if(!(p->flags & KICK_SENT))
		{
			if(reason) net.irc.send("KICK ", name, " ", p->nick, " :", reason, NULL);
			else net.irc.send("KICK ", name, " ", p->nick, " :", config.banreason, NULL);
			p->flags += KICK_SENT;
			ME.NextAction += AdditionalPenality();
		}
	}
	*/
	return 0;
}

int chan::invite(char *nick)
{
	if(!nick || !*nick) return 0;

	if(!getUser(nick))
	{
		char *a = strchr(nick, '!');
		if(a) *a = '\0';

		::invite.wisePush("INVITE ", nick, " ", name, NULL);
		if(penalty < 5) if(::invite.flush(&net.irc)) penalty += 3;
		return 1;
	}
	return 0;
}
